"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DownloadCenter = exports.validateConfig = exports.probePlatformDownloadLink = exports.validateDownloadLinks = exports.validateConfigSchema = void 0;
var util_1 = __importDefault(require("util"));
var node_fetch_1 = __importDefault(require("node-fetch"));
var ajv_1 = __importDefault(require("ajv"));
var s3_1 = __importDefault(require("aws-sdk/clients/s3"));
var download_center_config_schema_json_1 = __importDefault(require("./download-center-config.schema.json"));
var CONFIG_JSON_SCHEMA = Object.freeze(download_center_config_schema_json_1.default);
var ACL_PUBLIC_READ = 'public-read';
/**
 * Vaidates a download center configuration object against a json schema.
 * Throws an error if the configuration is invalid.
 *
 * @static
 * @param {DownloadCenterConfig} config -
 *  the download center product configuration document.
 * @memberof DownloadCenter
 */
function validateConfigSchema(config) {
    var ajv = new ajv_1.default();
    var validate = ajv.compile(CONFIG_JSON_SCHEMA);
    var valid = validate(config);
    if (!valid) {
        throw new Error("Invalid configuration: " + ajv.errorsText(validate.errors));
    }
}
exports.validateConfigSchema = validateConfigSchema;
/**
 * Validates all the asset links referenced in a configuration object.
 * Makes an HEAD http call for each asset link and throws an error in case
 * an asset is not reacheable.
 *
 * @static
 * @param {DownloadCenterConfig} config -
 *  the download center product configuration document.
 * @return {Promise<void>}
 * @memberof DownloadCenter
 */
function validateDownloadLinks(config) {
    return __awaiter(this, void 0, void 0, function () {
        var errors, platforms, _i, _a, version, _b, _c, platform, probes, errorMsg;
        return __generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    errors = {};
                    platforms = [];
                    for (_i = 0, _a = config.versions; _i < _a.length; _i++) {
                        version = _a[_i];
                        for (_b = 0, _c = version.platform; _b < _c.length; _b++) {
                            platform = _c[_b];
                            platforms.push(platform);
                        }
                    }
                    probes = platforms.map(function (platform) {
                        return probePlatformDownloadLink(platform)
                            .then(function (probe) {
                            if (!probe.ok) {
                                errors[platform.download_link] = probe.status;
                            }
                        });
                    });
                    return [4 /*yield*/, Promise.all(probes)];
                case 1:
                    _d.sent();
                    if (Object.keys(errors).length) {
                        errorMsg = Object.entries(errors).map(function (_a) {
                            var url = _a[0], status = _a[1];
                            return "- " + url + " -> " + status;
                        }).sort().join('\n');
                        throw new Error("Download center urls broken:\n" + errorMsg);
                    }
                    return [2 /*return*/];
            }
        });
    });
}
exports.validateDownloadLinks = validateDownloadLinks;
/**
 * Probes the download_link for a configuration platform.
 * Returns the response of the probe.
 *
 * @static
 * @param {DownloadCenterConfigPlatform} platform
 * @return {Promise<ProbeResponse>}
 * @memberof DownloadCenter
 */
function probePlatformDownloadLink(platform) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, node_fetch_1.default(platform.download_link, { method: 'HEAD' })];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports.probePlatformDownloadLink = probePlatformDownloadLink;
/**
 * Validates a download center config object.
 * Throws an error if the object has an incorrect format or if any of the
 * assets link is not reachable.
 *
 * @static
 * @param {DownloadCenterConfig} config -
 *  the download center product configuration document.
 * @return {Promise<void>}
 * @memberof DownloadCenter
 */
function validateConfig(config) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    validateConfigSchema(config);
                    return [4 /*yield*/, validateDownloadLinks(config)];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    });
}
exports.validateConfig = validateConfig;
var DownloadCenter = /** @class */ (function () {
    function DownloadCenter(bucketConfig) {
        var s3 = new s3_1.default(bucketConfig);
        this.s3GetObject = util_1.default.promisify(s3.getObject.bind(s3));
        this.s3Upload = util_1.default.promisify(s3.upload.bind(s3));
        this.s3BucketName = bucketConfig.bucket;
    }
    /**
     * Downloads an asset from the download center. This is equivalent to a
     * get object from the download center s3 bucket.
     *
     * @param {string} s3ObjectKey -
     *  the s3 object key of the asset that has to be downloaded,
     *  ie. `my-project/asset.zip`.
     * @return {(Promise<Content | undefined>)}
     * @memberof DownloadCenter
     */
    DownloadCenter.prototype.downloadAsset = function (s3ObjectKey) {
        return __awaiter(this, void 0, void 0, function () {
            var object;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!s3ObjectKey) {
                            throw new Error('s3ObjectKey is required');
                        }
                        return [4 /*yield*/, this.s3GetObject({
                                Key: s3ObjectKey,
                                Bucket: this.s3BucketName
                            })];
                    case 1:
                        object = _a.sent();
                        return [2 /*return*/, object.Body];
                }
            });
        });
    };
    /**
     * Uploads an asset to the download center. This is equivalent to an
     * upload to the download center s3 bucket.
     *
     * @param {string} s3ObjectKey -
     *  the s3 object key of the asset that has to be uploaded,
     *  ie. `my-project/asset.zip`.
     * @param {Content} content -
     *  a string, Buffer, Uint8Array, Blob or Readable containing the data of
     *  the asset to be uploaded.
     * @param {UploadAssetOptions} [options={}] -
     *  metadata for the upload.
     * @param {string} [options.contentType=undefined] -
     *  an optional content type of the asset. If not specified
     *  will be detected by s3.
     * @return {Promise<void>}
     * @memberof DownloadCenter
     */
    DownloadCenter.prototype.uploadAsset = function (s3ObjectKey, content, options) {
        if (options === void 0) { options = {}; }
        return __awaiter(this, void 0, void 0, function () {
            var uploadParams;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!s3ObjectKey) {
                            throw new Error('s3ObjectKey is required');
                        }
                        uploadParams = {
                            ACL: ACL_PUBLIC_READ,
                            Bucket: this.s3BucketName,
                            Key: s3ObjectKey,
                            Body: content,
                            ContentType: options.contentType
                        };
                        return [4 /*yield*/, this.s3Upload(uploadParams)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Downloads a product configuration from the download center bucket.
     *
     * @param {string} s3ObjectKey  -
     *  the s3 object key of the configuration that has to be downloaded,
     *  ie. `products/compass.json`.
     * @return {(Promise<DownloadCenterConfig | undefined>)} -
     *  the download center product configuration document.
     * @memberof DownloadCenter
     */
    DownloadCenter.prototype.downloadConfig = function (s3ObjectKey) {
        return __awaiter(this, void 0, void 0, function () {
            var body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.downloadAsset(s3ObjectKey)];
                    case 1:
                        body = _a.sent();
                        if (!body) {
                            return [2 /*return*/];
                        }
                        return [2 /*return*/, JSON.parse(body.toString())];
                }
            });
        });
    };
    /**
     * Validates and uploads a product configuration document
     * to the download center s3 bucket.
     *
     * @param {string} s3ObjectKey  -
     *  the s3 object key of the configuration that has to be uploaded,
     *  ie. `products/compass.json`.
     * @param {DownloadCenterConfig} config -
     *  the download center product configuration document.
     *
     * @return {Promise<void>}
     * @memberof DownloadCenter
     */
    DownloadCenter.prototype.uploadConfig = function (s3ObjectKey, config) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!s3ObjectKey) {
                            throw new Error('s3ObjectKey is required');
                        }
                        return [4 /*yield*/, validateConfig(config)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.uploadAsset(s3ObjectKey, JSON.stringify(config))];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    return DownloadCenter;
}());
exports.DownloadCenter = DownloadCenter;
//# sourceMappingURL=download-center.js.map