"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const autocomplete_1 = __importDefault(require("@mongosh/autocomplete"));
const errors_1 = require("@mongosh/errors");
const history_1 = require("@mongosh/history");
const shell_api_1 = require("@mongosh/shell-api");
const shell_evaluator_1 = require("@mongosh/shell-evaluator");
const types_1 = require("@mongosh/types");
const askcharacter_1 = __importDefault(require("askcharacter"));
const askpassword_1 = __importDefault(require("askpassword"));
const console_1 = require("console");
const events_1 = require("events");
const pretty_repl_1 = __importDefault(require("pretty-repl"));
const repl_1 = require("repl");
const stream_1 = require("stream");
const util_1 = require("util");
const asyncRepl = __importStar(require("./async-repl"));
const clr_1 = __importDefault(require("./clr"));
const constants_1 = require("./constants");
const format_output_1 = __importStar(require("./format-output"));
const js_multiline_to_singleline_1 = require("./js-multiline-to-singleline");
const line_by_line_input_1 = require("./line-by-line-input");
const log_entry_1 = require("./log-entry");
function fixupReplForNodeBug38314(repl) {
    {
        const input = new stream_1.PassThrough();
        const output = new stream_1.PassThrough();
        const evalFn = (code, ctx, filename, cb) => cb(new Error('err'));
        const prompt = 'prompt#';
        (0, repl_1.start)({ input, output, eval: evalFn, prompt });
        input.end('s\n');
        if (!String(output.read()).includes('prompt#prompt#')) {
            return;
        }
    }
    const domain = repl._domain;
    const domainErrorListeners = domain.listeners('error');
    const origListener = domainErrorListeners.find((fn) => fn.name === 'debugDomainError');
    if (!origListener) {
        throw new Error('Could not find REPL domain error listener');
    }
    domain.removeListener('error', origListener);
    domain.on('error', function (err) {
        const origDisplayPrompt = repl.displayPrompt;
        repl.displayPrompt = () => { };
        try {
            origListener.call(this, err);
        }
        finally {
            repl.displayPrompt = origDisplayPrompt;
        }
    });
}
class MongoshNodeRepl {
    constructor(options) {
        this.inspectCompact = 0;
        this.inspectDepth = 0;
        this.started = false;
        this.showStackTraces = false;
        this.loadNestingLevel = 0;
        this.redactHistory = 'remove';
        this.rawValueToShellResult = new WeakMap();
        this.input = options.input;
        this.lineByLineInput = new line_by_line_input_1.LineByLineInput(this.input);
        this.output = options.output;
        this.bus = options.bus;
        this.nodeReplOptions = options.nodeReplOptions || {};
        this.shellCliOptions = options.shellCliOptions || {};
        this.ioProvider = options.ioProvider;
        this.insideAutoCompleteOrGetPrompt = false;
        this._runtimeState = null;
    }
    setIsInteractive(value) {
        this.runtimeState().internalState.isInteractive = value;
    }
    async initialize(serviceProvider) {
        var _a, _b, _c;
        const internalState = new shell_api_1.ShellInternalState(serviceProvider, this.bus, this.shellCliOptions);
        const shellEvaluator = new shell_evaluator_1.ShellEvaluator(internalState, (value) => value);
        internalState.setEvaluationListener(this);
        await internalState.fetchConnectionInfo();
        let mongodVersion = (_a = internalState.connectionInfo.buildInfo) === null || _a === void 0 ? void 0 : _a.version;
        const apiVersion = (_c = (_b = serviceProvider.getRawClient()) === null || _b === void 0 ? void 0 : _b.serverApi) === null || _c === void 0 ? void 0 : _c.version;
        if (apiVersion) {
            mongodVersion = (mongodVersion ? mongodVersion + ' ' : '') + `(API Version ${apiVersion})`;
        }
        await this.greet(mongodVersion);
        await this.printStartupLog(internalState);
        this.inspectCompact = await this.getConfig('inspectCompact');
        this.inspectDepth = await this.getConfig('inspectDepth');
        this.showStackTraces = await this.getConfig('showStackTraces');
        const repl = asyncRepl.start({
            start: pretty_repl_1.default.start,
            input: this.lineByLineInput,
            output: this.output,
            prompt: '',
            writer: this.writer.bind(this),
            breakEvalOnSigint: true,
            preview: false,
            asyncEval: this.eval.bind(this),
            historySize: await this.getConfig('historyLength'),
            wrapCallbackError: (err) => Object.assign(new errors_1.MongoshInternalError(err.message), { stack: err.stack }),
            onAsyncSigint: this.onAsyncSigint.bind(this),
            ...this.nodeReplOptions
        });
        fixupReplForNodeBug38314(repl);
        const console = new console_1.Console({
            stdout: this.output,
            stderr: this.output,
            colorMode: this.getFormatOptions().colors
        });
        this.onClearCommand = console.clear.bind(console);
        repl.context.console = console;
        repl.context.Date = Date;
        this._runtimeState = {
            shellEvaluator,
            internalState,
            repl,
            console
        };
        const origReplCompleter = (0, util_1.promisify)(repl.completer.bind(repl));
        const mongoshCompleter = autocomplete_1.default.bind(null, internalState.getAutocompleteParameters());
        repl.completer =
            (0, util_1.callbackify)(async (text) => {
                this.insideAutoCompleteOrGetPrompt = true;
                try {
                    const [[replResults, replOrig], [mongoshResults, , mongoshResultsExclusive]] = await Promise.all([
                        (async () => await origReplCompleter(text) || [[]])(),
                        (async () => await mongoshCompleter(text))()
                    ]);
                    this.bus.emit('mongosh:autocompletion-complete');
                    if (mongoshResultsExclusive) {
                        return [mongoshResults, text];
                    }
                    const replResultPrefix = replOrig ? text.substr(0, text.lastIndexOf(replOrig)) : '';
                    const longReplResults = replResults.map((result) => replResultPrefix + result);
                    const deduped = [...new Set([...longReplResults, ...mongoshResults])];
                    return [deduped, text];
                }
                finally {
                    this.insideAutoCompleteOrGetPrompt = false;
                }
            });
        let originalHistory = null;
        const originalDisplayPrompt = repl.displayPrompt.bind(repl);
        repl.displayPrompt = (...args) => {
            if (!this.started) {
                return;
            }
            originalDisplayPrompt(...args);
            this.lineByLineInput.nextLine();
        };
        if (repl.commands.editor) {
            const originalEditorAction = repl.commands.editor.action.bind(repl);
            repl.commands.editor.action = (...args) => {
                originalHistory = [...repl.history];
                this.lineByLineInput.disableBlockOnNewline();
                return originalEditorAction(...args);
            };
        }
        repl.defineCommand('clear', {
            help: '',
            action: () => {
                repl.displayPrompt();
            }
        });
        repl.line = '';
        const historyFile = this.ioProvider.getHistoryFilePath();
        try {
            await (0, util_1.promisify)(repl.setupHistory).call(repl, historyFile);
            repl.on('flushHistory', () => {
                if (this.redactHistory !== 'keep') {
                    const history = repl.history;
                    (0, history_1.changeHistory)(history, this.redactHistory === 'remove-redact' ? 'redact-sensitive-data' : 'keep-sensitive-data');
                }
            });
            repl.on(asyncRepl.evalFinish, (ev) => {
                if (this.insideAutoCompleteOrGetPrompt) {
                    return;
                }
                const history = repl.history;
                if (ev.success === false && ev.recoverable) {
                    if (originalHistory === null) {
                        originalHistory = history.slice(1);
                    }
                }
                else if (originalHistory !== null) {
                    const newHistoryEntry = (0, js_multiline_to_singleline_1.makeMultilineJSIntoSingleLine)(ev.input);
                    if (newHistoryEntry.length > 0) {
                        originalHistory.unshift(newHistoryEntry);
                    }
                    history.splice(0, history.length, ...originalHistory);
                    originalHistory = null;
                }
            });
        }
        catch (err) {
            const warn = new errors_1.MongoshWarning('Error processing history file: ' + err.message);
            this.output.write(this.writer(warn) + '\n');
        }
        repl.on(asyncRepl.evalFinish, () => {
            this.bus.emit('mongosh:evaluate-finished');
        });
        repl.on('exit', async () => {
            try {
                await this.onExit();
            }
            catch (_a) { }
        });
        internalState.setCtx(repl.context);
        return { __initialized: 'yes' };
    }
    async startRepl(_initializationToken) {
        this.started = true;
        const { repl } = this.runtimeState();
        this.lineByLineInput.start();
        this.input.resume();
        repl.setPrompt(await this.getShellPrompt());
        repl.displayPrompt();
    }
    async greet(mongodVersion) {
        if (this.shellCliOptions.quiet) {
            return;
        }
        const { version } = require('../package.json');
        let text = '';
        if (!this.shellCliOptions.nodb) {
            text += `Using MongoDB:\t\t${mongodVersion}\n`;
        }
        text += `${this.clr('Using Mongosh', ['bold', 'yellow'])}:\t\t${version}\n`;
        text += `${constants_1.MONGOSH_WIKI}\n`;
        if (!await this.getConfig('disableGreetingMessage')) {
            text += `${constants_1.TELEMETRY_GREETING_MESSAGE}\n`;
            await this.setConfig('disableGreetingMessage', true);
        }
        this.output.write(text);
    }
    async printStartupLog(internalState) {
        if (this.shellCliOptions.nodb || this.shellCliOptions.quiet) {
            return;
        }
        let result;
        try {
            result = await internalState.currentDb.adminCommand({ getLog: 'startupWarnings' });
            if (!result) {
                throw new errors_1.MongoshCommandFailed('adminCommand getLog unexpectedly returned no result');
            }
        }
        catch (error) {
            this.bus.emit('mongosh:error', error, 'repl');
            return;
        }
        if (!result.log || !result.log.length) {
            return;
        }
        let text = '';
        text += `${this.clr('------', ['bold', 'yellow'])}\n`;
        text += `   ${this.clr('The server generated these startup warnings when booting:', ['bold', 'yellow'])}\n`;
        result.log.forEach(logLine => {
            try {
                const entry = (0, log_entry_1.parseAnyLogEntry)(logLine);
                text += `   ${entry.timestamp}: ${entry.message}\n`;
            }
            catch (e) {
                text += `   Unexpected log line format: ${logLine}\n`;
            }
        });
        text += `${this.clr('------', ['bold', 'yellow'])}\n`;
        text += '\n';
        this.output.write(text);
    }
    async eval(originalEval, input, context, filename) {
        if (!this.insideAutoCompleteOrGetPrompt) {
            this.lineByLineInput.enableBlockOnNewLine();
        }
        const { repl, shellEvaluator } = this.runtimeState();
        let interrupted = false;
        try {
            const rawValue = await shellEvaluator.customEval(originalEval, input, context, filename);
            if ((typeof rawValue === 'object' && rawValue !== null) || typeof rawValue === 'function') {
                this.rawValueToShellResult.set(rawValue, await (0, shell_api_1.toShellResult)(rawValue));
            }
            if (!this.insideAutoCompleteOrGetPrompt || (0, shell_api_1.getShellApiType)(rawValue) === null) {
                return rawValue;
            }
            return Object.fromEntries(Object.entries(rawValue)
                .filter(([key]) => !key.startsWith('_')));
        }
        catch (err) {
            if (this.runtimeState().internalState.interrupted.isSet()) {
                interrupted = true;
                this.bus.emit('mongosh:eval-interrupted');
                const result = {
                    type: null,
                    rawValue: undefined,
                    printable: undefined
                };
                return result;
            }
            if (!isErrorLike(err)) {
                throw new Error(this.formatOutput({
                    value: err
                }));
            }
            throw err;
        }
        finally {
            if (!this.insideAutoCompleteOrGetPrompt && !interrupted) {
                repl.setPrompt(await this.getShellPrompt());
            }
            if (this.loadNestingLevel <= 1) {
                this.bus.emit('mongosh:eval-complete');
            }
        }
    }
    async onLoad(filename) {
        const { contents, absolutePath } = await this.ioProvider.readFileUTF8(filename);
        return {
            resolvedFilename: absolutePath,
            evaluate: async () => {
                this.loadNestingLevel += 1;
                try {
                    await this.loadExternalCode(contents, absolutePath);
                }
                finally {
                    this.loadNestingLevel -= 1;
                }
            }
        };
    }
    async loadExternalFile(filename) {
        await this.runtimeState().internalState.shellApi.load(filename);
    }
    async loadExternalCode(code, filename) {
        const { repl } = this.runtimeState();
        return await (0, util_1.promisify)(repl.eval.bind(repl))(code, repl.context, filename);
    }
    async onAsyncSigint() {
        var _a;
        const { internalState } = this.runtimeState();
        if (internalState.interrupted.isSet()) {
            return true;
        }
        this.output.write('Stopping execution...');
        const mongodVersion = (_a = internalState.connectionInfo.buildInfo) === null || _a === void 0 ? void 0 : _a.version;
        if (mongodVersion === null || mongodVersion === void 0 ? void 0 : mongodVersion.match(/^(4\.0\.|3\.)\d+/)) {
            this.output.write(this.clr(`\nWARNING: Operations running on the server cannot be killed automatically for MongoDB ${mongodVersion}.` +
                '\n         Please make sure to kill them manually. Killing operations is supported starting with MongoDB 4.1.', ['bold', 'yellow']));
        }
        const fullyInterrupted = await internalState.onInterruptExecution();
        await Promise.race([
            (0, events_1.once)(this.bus, 'mongosh:eval-interrupted'),
            new Promise(setImmediate)
        ]);
        const fullyResumed = await internalState.onResumeExecution();
        if (!fullyInterrupted || !fullyResumed) {
            this.output.write(this.formatError({
                name: 'MongoshInternalError',
                message: 'Could not re-establish all connections, we suggest to restart the shell.'
            }));
        }
        this.bus.emit('mongosh:interrupt-complete');
        const { repl } = this.runtimeState();
        repl.setPrompt(await this.getShellPrompt());
        return true;
    }
    writer(result) {
        var _a;
        if (isErrorLike(result)) {
            const output = {
                ...result,
                message: result.message || result.errmsg,
                name: result.name || 'MongoshInternalError',
                stack: result.stack
            };
            this.bus.emit('mongosh:error', output, 'repl');
            return this.formatError(output);
        }
        return this.formatShellResult((_a = this.rawValueToShellResult.get(result)) !== null && _a !== void 0 ? _a : { type: null, printable: result });
    }
    formatShellResult(result) {
        return this.formatOutput({ type: result.type, value: result.printable });
    }
    onPrint(values) {
        const joined = values.map((value) => this.formatShellResult(value)).join(' ');
        this.output.write(joined + '\n');
    }
    async onPrompt(question, type) {
        var _a;
        if (type === 'password') {
            const passwordPromise = (0, askpassword_1.default)({
                input: this.input,
                output: this.output,
                replacementCharacter: '*'
            });
            this.output.write(question + '\n');
            return (await passwordPromise).toString();
        }
        else if (type === 'yesno') {
            let result = '';
            while (true) {
                const charPromise = (0, askcharacter_1.default)({
                    input: this.input,
                    output: this.output
                });
                this.output.write(question + ': ');
                result = await charPromise;
                if (result.length > 0 && !result.match(/^[yYnN\r\n]$/)) {
                    this.output.write('\nPlease enter Y or N: ');
                }
                else {
                    break;
                }
            }
            this.output.write('\n');
            return (_a = { 'Y': 'yes', 'N': 'no' }[result.toUpperCase()]) !== null && _a !== void 0 ? _a : '';
        }
        throw new Error(`Unrecognized prompt type ${type}`);
    }
    formatOutput(value) {
        return (0, format_output_1.default)(value, this.getFormatOptions());
    }
    formatError(value) {
        return (0, format_output_1.formatError)(value, this.getFormatOptions());
    }
    clr(text, style) {
        return (0, clr_1.default)(text, style, this.getFormatOptions());
    }
    getFormatOptions() {
        var _a, _b, _c, _d, _e;
        const output = this.output;
        return {
            colors: (_c = (_b = (_a = this._runtimeState) === null || _a === void 0 ? void 0 : _a.repl) === null || _b === void 0 ? void 0 : _b.useColors) !== null && _c !== void 0 ? _c : (output.isTTY && output.getColorDepth() > 1),
            compact: this.inspectCompact,
            depth: this.inspectDepth,
            showStackTraces: this.showStackTraces,
            bugReportErrorMessageInfo: (_e = (_d = this.ioProvider).bugReportErrorMessageInfo) === null || _e === void 0 ? void 0 : _e.call(_d)
        };
    }
    runtimeState() {
        if (this._runtimeState === null) {
            throw new errors_1.MongoshInternalError('Mongosh not started yet');
        }
        return this._runtimeState;
    }
    async close() {
        const rs = this._runtimeState;
        if (rs) {
            this._runtimeState = null;
            rs.repl.close();
            await rs.internalState.close(true);
            if (this.output.writableLength > 0) {
                await (0, events_1.once)(this.output, 'drain');
            }
        }
    }
    async onExit(exitCode) {
        await this.close();
        return this.ioProvider.exit(exitCode);
    }
    async getConfig(key) {
        return this.ioProvider.getConfig(key);
    }
    async setConfig(key, value) {
        const result = await this.ioProvider.setConfig(key, value);
        if (result === 'success') {
            if (key === 'historyLength' && this._runtimeState) {
                this.runtimeState().repl.historySize = value;
            }
            if (key === 'inspectCompact') {
                this.inspectCompact = value;
            }
            if (key === 'inspectDepth') {
                this.inspectDepth = value;
            }
            if (key === 'showStackTraces') {
                this.showStackTraces = value;
            }
            if (key === 'redactHistory') {
                this.redactHistory = value;
            }
        }
        return result;
    }
    async validateConfig(key, value) {
        return types_1.CliUserConfigValidator.validate(key, value);
    }
    listConfigOptions() {
        return this.ioProvider.listConfigOptions();
    }
    async startMongocryptd() {
        return this.ioProvider.startMongocryptd();
    }
    async getShellPrompt() {
        const { repl, internalState } = this.runtimeState();
        try {
            this.insideAutoCompleteOrGetPrompt = true;
            if (typeof repl.context.prompt !== 'undefined') {
                const promptResult = await this.loadExternalCode(`
        (() => {
          switch (typeof prompt) {
            case 'function':
              return prompt();
            case 'string':
              return prompt;
          }
        })()`, '<prompt loader>');
                if (typeof promptResult === 'string') {
                    return promptResult;
                }
            }
        }
        catch (_a) {
        }
        finally {
            this.insideAutoCompleteOrGetPrompt = false;
        }
        try {
            return await internalState.getDefaultPrompt();
        }
        catch (_b) {
        }
        return '> ';
    }
}
function isErrorLike(value) {
    try {
        return value && (0, shell_api_1.getShellApiType)(value) === null && ((value.message !== undefined && typeof value.stack === 'string') ||
            (value.code !== undefined && value.errmsg !== undefined));
    }
    catch (err) {
        throw new errors_1.MongoshInternalError((err === null || err === void 0 ? void 0 : err.message) || String(err));
    }
}
exports.default = MongoshNodeRepl;
//# sourceMappingURL=mongosh-repl.js.map