"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runSmokeTests = void 0;
const child_process_1 = require("child_process");
const assert_1 = __importDefault(require("assert"));
const events_1 = require("events");
const history_1 = require("@mongosh/history");
const smoke_tests_fle_1 = __importDefault(require("./smoke-tests-fle"));
async function runSmokeTests(smokeTestServer, executable, ...args) {
    console.log('MONGOSH_SMOKE_TEST_SERVER set?', !!smokeTestServer);
    if (process.env.IS_CI) {
        (0, assert_1.default)(!!smokeTestServer, 'Make sure MONGOSH_SMOKE_TEST_SERVER is set in CI');
    }
    for (const { input, output, testArgs } of [{
            input: 'print("He" + "llo" + " Wor" + "ld!")',
            output: /Hello World!/,
            testArgs: ['--nodb'],
        }].concat(smokeTestServer ? [{
            input: `
      const dbname = "testdb_simplesmoke" + new Date().getTime();
      use(dbname);
      db.testcoll.insertOne({ d: new Date() });
      if (Object.keys(EJSON.serialize(db.testcoll.findOne()).d)[0] === '$date') {
        print('Test succeeded');
      }
      db.dropDatabase();`,
            output: /Test succeeded/,
            testArgs: [smokeTestServer]
        }, {
            input: smoke_tests_fle_1.default,
            output: /Test succeeded|Test skipped/,
            testArgs: [smokeTestServer]
        }] : [])) {
        await runSmokeTest(executable, [...args, ...testArgs], input, output);
    }
    console.log('all tests passed');
}
exports.runSmokeTests = runSmokeTests;
async function runSmokeTest(executable, args, input, output) {
    const proc = (0, child_process_1.spawn)(executable, [...args], {
        stdio: ['pipe', 'pipe', 'inherit']
    });
    let stdout = '';
    proc.stdout.setEncoding('utf8').on('data', (chunk) => { stdout += chunk; });
    proc.stdin.end(input);
    await (0, events_1.once)(proc.stdout, 'end');
    try {
        assert_1.default.match(stdout, output);
        console.error({ status: 'success', input, output, stdout, executable, args: args.map(history_1.redactURICredentials) });
    }
    catch (err) {
        console.error({ status: 'failure', input, output, stdout, executable, args: args.map(history_1.redactURICredentials) });
        throw err;
    }
}
//# sourceMappingURL=smoke-tests.js.map