/// <reference types="node" />
import { ShellApiWithMongoClass } from './decorators';
import { ClientEncryption as MongoCryptClientEncryption, ClientEncryptionDataKeyProvider, ClientEncryptionEncryptOptions, KMSProviders, AWSEncryptionKeyOptions, AzureEncryptionKeyOptions, GCPEncryptionKeyOptions } from '@mongosh/service-provider-core';
import type { Document, BinaryType } from '@mongosh/service-provider-core';
import Cursor from './cursor';
import { DeleteResult } from './result';
import { asPrintable } from './enums';
import type Mongo from './mongo';
export declare type ClientSideFieldLevelEncryptionKmsProvider = Omit<KMSProviders, 'local'> & {
    local?: {
        key: Buffer | string | BinaryType;
    };
};
export interface ClientSideFieldLevelEncryptionOptions {
    keyVaultClient?: Mongo;
    keyVaultNamespace: string;
    kmsProviders: ClientSideFieldLevelEncryptionKmsProvider;
    schemaMap?: Document;
    bypassAutoEncryption?: boolean;
    explicitEncryptionOnly?: boolean;
}
export declare class ClientEncryption extends ShellApiWithMongoClass {
    _mongo: Mongo;
    _libmongocrypt: MongoCryptClientEncryption;
    constructor(mongo: Mongo);
    [asPrintable](): string;
    encrypt(encryptionId: BinaryType, value: any, encryptionAlgorithm: ClientEncryptionEncryptOptions['algorithm']): Promise<BinaryType>;
    decrypt(encryptedValue: BinaryType): Promise<any>;
}
export declare class KeyVault extends ShellApiWithMongoClass {
    _mongo: Mongo;
    _clientEncryption: ClientEncryption;
    private _keyColl;
    constructor(clientEncryption: ClientEncryption);
    [asPrintable](): string;
    createKey(kms: 'local', keyAltNames?: string[]): Promise<Document>;
    createKey(kms: ClientEncryptionDataKeyProvider, legacyMasterKey: string, keyAltNames?: string[]): Promise<Document>;
    createKey(kms: ClientEncryptionDataKeyProvider, options: AWSEncryptionKeyOptions | AzureEncryptionKeyOptions | GCPEncryptionKeyOptions | undefined): Promise<Document>;
    createKey(kms: ClientEncryptionDataKeyProvider, options: AWSEncryptionKeyOptions | AzureEncryptionKeyOptions | GCPEncryptionKeyOptions | undefined, keyAltNames: string[]): Promise<Document>;
    getKey(keyId: BinaryType): Promise<Cursor>;
    getKeyByAltName(keyAltName: string): Promise<Cursor>;
    getKeys(): Promise<Cursor>;
    deleteKey(keyId: BinaryType): Promise<DeleteResult | Document>;
    addKeyAlternateName(keyId: BinaryType, keyAltName: string): Promise<Document>;
    removeKeyAlternateName(keyId: BinaryType, keyAltName: string): Promise<Document>;
}
