import { AutoEncryptionOptions, ConnectInfo, ServerApi, ServiceProvider } from '@mongosh/service-provider-core';
import type { ApiEvent, ConfigProvider, MongoshBus, ShellUserConfig } from '@mongosh/types';
import ChangeStreamCursor from './change-stream-cursor';
import { Topologies } from './enums';
import { AggregationCursor, Cursor, Database, Mongo, ShellApi, ShellResult } from './index';
import { InterruptFlag } from './interruptor';
export interface ShellCliOptions {
    nodb?: boolean;
    mongocryptdSpawnPath?: string;
}
export interface AutocompleteParameters {
    topology: () => Topologies;
    apiVersionInfo: () => Required<ServerApi> | undefined;
    connectionInfo: () => ConnectInfo | undefined;
    getCollectionCompletionsForCurrentDb: (collName: string) => Promise<string[]>;
    getDatabaseCompletions: (dbName: string) => Promise<string[]>;
}
export interface OnLoadResult {
    resolvedFilename: string;
    evaluate(): Promise<void>;
}
export interface EvaluationListener extends Partial<ConfigProvider<ShellUserConfig>> {
    onPrint?: (value: ShellResult[]) => Promise<void> | void;
    onPrompt?: (question: string, type: 'password' | 'yesno') => Promise<string> | string;
    onClearCommand?: () => Promise<void> | void;
    onExit?: (exitCode?: number) => Promise<never>;
    onLoad?: (filename: string) => Promise<OnLoadResult> | OnLoadResult;
    startMongocryptd?: () => Promise<AutoEncryptionOptions['extraOptions']>;
}
export interface ShellPlugin {
    transformError?: (err: Error) => Error;
}
export default class ShellInternalState {
    currentCursor: Cursor | AggregationCursor | ChangeStreamCursor | null;
    currentDb: Database;
    messageBus: MongoshBus;
    initialServiceProvider: ServiceProvider;
    connectionInfo: any;
    context: any;
    mongos: Mongo[];
    shellApi: ShellApi;
    shellBson: any;
    cliOptions: ShellCliOptions;
    evaluationListener: EvaluationListener;
    mongocryptdSpawnPath: string | null;
    displayBatchSizeFromDBQuery: number | undefined;
    isInteractive: boolean;
    readonly interrupted: InterruptFlag;
    resumeMongosAfterInterrupt: Array<{
        mongo: Mongo;
        resume: (() => Promise<void>) | null;
    }> | undefined;
    private plugins;
    private alreadyTransformedErrors;
    constructor(initialServiceProvider: ServiceProvider, messageBus?: any, cliOptions?: ShellCliOptions);
    fetchConnectionInfo(): Promise<void>;
    close(force: boolean): Promise<void>;
    setDbFunc(newDb: any): Database;
    setCtx(contextObject: any): void;
    get currentServiceProvider(): ServiceProvider;
    emitApiCall(event: ApiEvent): void;
    emitDeprecatedApiCall(event: ApiEvent): void;
    setEvaluationListener(listener: EvaluationListener): void;
    getAutocompleteParameters(): AutocompleteParameters;
    apiVersionInfo(): Required<ServerApi> | undefined;
    onInterruptExecution(): Promise<boolean>;
    onResumeExecution(): Promise<boolean>;
    getDefaultPrompt(): Promise<string>;
    private getDefaultPromptPrefix;
    private getTopologySpecificPrompt;
    private getTopologySinglePrompt;
    registerPlugin(plugin: ShellPlugin): void;
    transformError(err: any): any;
}
